# encoding: utf-8

#*    dsgui - a graphical user interface for Datove schranky
#*    Copyright (C) 2009-2010  CZ.NIC, z.s.p.o. (http://www.nic.cz)
#*
#*    This library is free software; you can redistribute it and/or
#*    modify it under the terms of the GNU Library General Public
#*    License as published by the Free Software Foundation; either
#*    version 2 of the License, or (at your option) any later version.
#*
#*    This library is distributed in the hope that it will be useful,
#*    but WITHOUT ANY WARRANTY; without even the implied warranty of
#*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#*    Library General Public License for more details.
#*
#*    You should have received a copy of the GNU Library General Public
#*    License along with this library; if not, write to the Free
#*    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

"""
Here resides a separate dialog for finding Data Boxes
"""

import pygtk
import gtk
import sys
pygtk.require("2.0")

class CredentialsDialog(object):
  
  # maps login_method to a set of widget names
  widget_groups = {"username": ('password_entry','password_label'),
                   "certificate": ('p12file_label','p12file_chooserbutton'),
                  } 
  
  def __init__(self, parent, credentials=None):
    self.parent = parent
    self.builder = gtk.Builder()
    gui_xml = self.parent.fix_ui_xml("credentials_dialog.ui")
    try:
      self.builder.add_from_string(gui_xml.decode('utf-8'))
    except:
      # this fixes problem with different API on windows and PyGTK 2.12
      self.builder.add_from_string(gui_xml.decode('utf-8'), -1)
    self.builder.connect_signals(self)
    self.warning_label = self.builder.get_object("warning_label")
    self.dialog = self.builder.get_object("credentials_dialog")
    # fill in the data from credentials
    login_entry = self.builder.get_object("login_entry")
    if credentials:
      login_entry.set_text(credentials.username or "")
      login_entry.set_sensitive(False)
    password_entry = self.builder.get_object("password_entry")
    password_entry.set_text(credentials and credentials.password or "")
    test_account_button = self.builder.get_object("test_environment_checkbutton")
    if credentials and credentials.test_account == True:
      # make normal account the default 
      test_account_button.set_active(True)
    else:
      test_account_button.set_active(False)
    if credentials:
      self.p12file = credentials.p12file
    else:
      self.p12file = None
    if self.p12file:
      self.builder.get_object('p12file_chooserbutton').\
          set_label(self.p12file)
    # check if certificate login is available
    from dslib.client import Client
    CredentialsDialog.CERT_LOGIN_AVAILABLE = Client.CERT_LOGIN_AVAILABLE
    # select the proper login method and force callbacks on the rest
    if not credentials or (credentials and credentials.login_method == 'username'):
      self.builder.get_object('username_login_radiobutton').set_active(True)
      self.builder.get_object('certificate_login_radiobutton').toggled()
    else:
      if self.CERT_LOGIN_AVAILABLE: 
        self.builder.get_object('certificate_login_radiobutton').set_active(True)
        self.builder.get_object('username_login_radiobutton').toggled()
      else:
        self.builder.get_object('username_login_radiobutton').set_active(True)
        self.builder.get_object('certificate_login_radiobutton').toggled()
    if not self.CERT_LOGIN_AVAILABLE:
      # deactivate the certificate option
      self.builder.get_object('certificate_login_radiobutton').set_sensitive(False)
      self.warning_label.set_text(_("""Certificate login is not available -
 the PyOpenSSL package could not be loaded."""))

  def run(self):
    result = self.dialog.run()
    return result

  def destroy(self):
    self.dialog.destroy()

  def hide(self):
    self.dialog.hide()
    
  def show_all(self):
    self.dialog.show_all()

  # callback handlers
  
  def on_certificate_login_radiobutton_toggled(self, w):
    self._set_widget_group_sensitivity('certificate', sensitive=w.get_active())
    self._update_warning()

  def on_username_login_radiobutton_toggled(self, w):
    self._set_widget_group_sensitivity('username', sensitive=w.get_active())
    self._update_warning()
    
  def on_p12file_chooserbutton_clicked(self, w):
    new = self._ask_a_certificate_filename(
                    self.p12file,
                    _("Select a certificate file in PKCS12 format")
                    )
    self.p12file = new
    self.builder.get_object('p12file_chooserbutton').\
                    set_label(self.p12file or _("(None)"))    
    self._update_warning()
  
 
  def _ask_a_certificate_filename(self, init_value, title):
    if not init_value:
      value = "(None)"
    else:
      value = init_value
    d = gtk.FileChooserDialog(title=title,
                              parent=self.dialog,
                              action=gtk.FILE_CHOOSER_ACTION_OPEN,
                              buttons=(gtk.STOCK_OK,gtk.RESPONSE_OK,
                                       gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL)
                              )
    f2 = gtk.FileFilter()
    f2.set_name(_("PKCS12 files (*.p12)"))
    f2.add_pattern("*.p12")
    d.add_filter(f2)
    f1 = gtk.FileFilter()
    f1.set_name(_("All files (*)"))
    f1.add_pattern("*")
    d.add_filter(f1)
    d.set_filename(value)
    d.set_filter(f2) # I have no idea why this does not in fact work :(
    ok = d.run()
    if ok == gtk.RESPONSE_OK:
      fullname = d.get_filename()
      d.destroy()
      return fullname.decode(sys.getfilesystemencoding())
    else:
      d.destroy()
      return init_value
  
  def _set_widget_group_sensitivity(self, widget_group, sensitive=True):
    for widget_id in self.widget_groups[widget_group]:
        self.builder.get_object(widget_id).set_sensitive(sensitive)

  def _update_warning(self):
    if self.builder.get_object('certificate_login_radiobutton').get_active():
      if not self.p12file:
        # some file is missing
        self.warning_label.set_text(_("You must supply the certificate file!"))
        self.builder.get_object("ok_button").set_sensitive(False)
      else:
        self.warning_label.set_text("")
        self.builder.get_object("ok_button").set_sensitive(True)
    else:
      self.warning_label.set_text("")
      self.builder.get_object("ok_button").set_sensitive(True)

  # end of callbacks
  
  def get_data(self):
    """returns a dictionary with filled-in data"""
    # login method
    if self.builder.get_object('username_login_radiobutton').get_active():
      login_method = 'username'
    else:
      login_method = 'certificate'
    username = self.builder.get_object('login_entry').get_text()
    password = self.builder.get_object('password_entry').get_text()
    p12file = self.p12file
    test_account = self.builder.get_object('test_environment_checkbutton').get_active()
    return dict(login_method=login_method,
                username=username,
                password=password,
                p12file=p12file,
                test_account=test_account)
    
  def set_main_label_text(self, text):
    self.builder.get_object("credentials_dialog_main_label").set_text(text)
