#*    dsgui - a graphical user interface for Datove schranky
#*    Copyright (C) 2009-2010  CZ.NIC, z.s.p.o. (http://www.nic.cz)
#*
#*    This library is free software; you can redistribute it and/or
#*    modify it under the terms of the GNU Library General Public
#*    License as published by the Free Software Foundation; either
#*    version 2 of the License, or (at your option) any later version.
#*
#*    This library is distributed in the hope that it will be useful,
#*    but WITHOUT ANY WARRANTY; without even the implied warranty of
#*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#*    Library General Public License for more details.
#*
#*    You should have received a copy of the GNU Library General Public
#*    License along with this library; if not, write to the Free
#*    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USAimport tempfile

import tempfile
import os
import sys
from threading import Thread
import time

DATE_TIME_FORMAT = "%Y-%m-%d %H:%M"

def create_temp_file(ext, content):
    temp = tempfile.mkstemp(suffix=ext)
    os.write(temp[0], content)
    os.close(temp[0])
    return temp[1]

def open_file_in_associated_app(fname, content):
  done = False
  ext = os.path.splitext(fname)[1]
  if os.name == 'nt':
    # WINDOWS
    temp_file = create_temp_file(ext, content)
    try:
      os.startfile(temp_file)
      done = True
    except WindowsError:
      done = False
  else:
    # we will use an external command via subprocess
    try:
      import gnomevfs
    except ImportError:
      GNOMEVFS_AVAILABLE = False
    else:
      GNOMEVFS_AVAILABLE = True
    command = None
    if GNOMEVFS_AVAILABLE:
      # GNOME
      mime_type = gnomevfs.get_mime_type_for_data(content)
      mime_app = gnomevfs.mime_get_default_application(mime_type)
      if mime_app:
        command = mime_app[2]
    elif os.environ.get('KDE_FULL_SESSION') == 'true':
      # KDE
      command = 'kde-open'
    elif sys.platform == 'darwin':
      command = 'open'
    # did we find anything?
    if command:
      import subprocess
      try:
        temp_file = create_temp_file(ext, content)
        p = subprocess.Popen([command,temp_file], close_fds=True)
        t = MonitorThread(p)
        t.start()
      except:
        pass
      else:
        done = True
  return done
   
def get_home_dir(topdir):
  dir = None
  if os.name == "nt":
    try:
      import _winreg as reg
      dir = reg.QueryValueEx(reg.OpenKey(reg.HKEY_CURRENT_USER,
                                         "Volatile Environment"),
                                         "APPDATA")[0]
    except EnvironmentError:
      pass
  if not dir:
    dir = os.getenv('HOME') or topdir
  return dir

def format_datetime(d):
  """formats a datetime according to application wide settings"""
  #strftime(locale.nl_langinfo(locale.D_T_FMT))
  if d:
    return d.strftime(DATE_TIME_FORMAT)
  return ""



class MonitorThread(Thread):
  """this thread monitors a child process until it dies, so that it does not
  become a zombie"""
  
  def __init__(self, process):
    Thread.__init__(self)
    self.process = process

  def run(self):
    while self.process.poll() == None:
      time.sleep(2)

